% $Id: feature_ordering.pl,v 1.10 2003/05/08 16:39:18 dm Exp $
%
% This file defines code for ordering features in feature structures
% that are displayed.
%
% Author: Detmar Meurers
%
% The user specifies feature order as follows:
%
% f <<< g.    %% meaning: f will be ordered before g.
% <<< h.      %% meaning: h will be ordered last.
% >>> i.      %% meaning: i will be ordered first. 
%
% Note: These statements are identical to the ones that were
% used by the ConTroll system. The ConTroll feature_ordering code
% was buggy though, so this code is rewritten from scratch.
%
% Added feature hiding based on hidden_feat/1 statements in the grammar
% or the standard ale commands no_write_feat/1.

:- use_module(library(lists),[member/2]).
:- use_module(library(ugraphs),[transitive_closure/2]).

:- op(850,xfy,<<<).
:- op(850,fx,<<<).
:- op(850,fx,>>>).

:- dynamic feature_order/2, feature_order_first/1, feature_order_last/1.

order_features(Fs0,Fs) :-
   t_samsort(feat_order_rel,Fs0,Fs).

% initialization routines -----

% clear_feature_output/0  abolishes source declarations
clear_feature_output :-
   abolish(hidden_feat/1),
   abolish((<<<)/1),
   abolish((<<<)/2),
   abolish((>>>)/1).

process_feature_output :-
   process_feature_hiding,
   process_feature_order.

process_feature_hiding :-
   retractall(no_write_feat_flag(_)),
   initialize_feature_hiding.

process_feature_order :-
   retractall(feature_order(_,_)),
   retractall(feature_order_first(_)),
   retractall(feature_order_last(_)),
   initialize_feature_order.

% -----

initialize_feature_hiding :-
   fail_if_undefined(hidden_feat(F)),
   no_write_feat(F),
   fail;
   true.

initialize_feature_order :-
   % Deal with the binary order specifications:
   (   % get a graph G from user specified order predicates
       setof(X-YList,setof(Y,feat_order_spec(X,Y),YList),G)
   ->  % compute transitive closure of G
       transitive_closure(G,TransG),
       assert_trans_feat_ord(TransG)
   ;   true			% no feature order specified
   ),
   % Transfer the first and last specifications into an asserted
   % internal representation, which allows us to check for errors here.
   (  feat_first_spec(F),
      assert(feature_order_first(F)),
      fail
   ;
      true
   ),
   (  feat_last_spec(F),
      assert(feature_order_first(F)),
      fail
   ;
      true
   ).
   
   
  
assert_trans_feat_ord([]).
assert_trans_feat_ord([X-L|Rest]) :-
        member(Y,L),
	assert(feature_order(X,Y)),
	fail
	;
	assert_trans_feat_ord(Rest).

% -----

feat_order_rel(fval(X,_,_),fval(Y,_,_)) :-
   feat_order_rel_aux(X,Y).

% Feature order relation:
feat_order_rel_aux(X,_) :-	% enforce feat_first order statement
   feature_order_first(X),
   !.
feat_order_rel_aux(_,Y) :-	% enforce feat_last order statement
   feature_order_last(Y),
   !.
feat_order_rel_aux(X,Y) :-	% enforce feature order statement
   \+ feature_order_first(Y),	% unless conflicting with feat_first
   \+ feature_order_last(X),	%     or conflicting with feat_last
   feature_order(X,Y),
   !.
feat_order_rel_aux(X,Y) :-	% if it doesn't contradict
   \+ feature_order_first(Y),	% - feat_first statement
   \+ feature_order_last(X),	% - feat_last statement
   \+ feature_order(Y,_),	% - captures contradiction with Y < X
				%   as well as breaking transitivity chain
   X @=< Y.			% then enforce standard term order

% The breaking of the transitivity chain occurs at last element, e.g.
% if we have features a, b, c, and d  and the statement d <<< a in the
% grammar, then we get d < a < b < c since a @=<b @=< c but since d <<< _
% is in the grammar the \+ feature_order(Y,_) stops us from enforcing c @=< d. 

% -----

% predicates for looking up the user specifications:

feat_order_spec(X,Y) :-
   fail_if_undefined(X<<<Y),
   (  feature(X),feature(Y)
   -> true
   ;  (  feature(X)
      -> format(user_error,'~N**Warning: ignoring feature ordering "~w" for unintroduced feature "~w".~n',[X<<<Y,Y]),fail
      ;  (  feature(Y)
	 -> format(user_error,'~N**Warning: ignoring feature ordering "~w" for unintroduced feature "~w".~n',[X<<<Y,X]),fail
	 ;  format(user_error,'~N**Warning: ignoring feature ordering "~w" for unintroduced features "~w" and "~w".~n',[X<<<Y,X,Y]),fail
	 )
      )
   ).

feat_first_spec(First) :-
   fail_if_undefined((>>> First)),
   (  feature(First)
   -> true
   ;  format(user_error,'~N**Warning: ignoring feature ordering "~w" for unintroduced feature "~w".~n',[>>>First,First]),fail
   ).

feat_last_spec(Last) :-
   fail_if_undefined((<<< Last)),
   (  feature(Last)
   -> true
   ;  format(user_error,'~N**Warning: ignoring feature ordering "~w" for unintroduced feature "~w".~n',[<<<Last,Last]),fail
   ).


%% -*-Mode: prolog;-*-

/* % some useful predicates that currently aren't used:

% sister_feats(+Feat,list(Feats): given a feature, returns the list of 
% other features appropriate for the type or its subtypes
sister_feats(F,L) :-
   setof(SisterF,T^T1^T2^(approp(F,T,T1),approp(SisterF,T,T2)),L).

% list of all features:
all_feats(AllFeats) :-
   setof(F,T0^T^approp(F,T0,T),AllFeats).
*/
