/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#include "LRS_MainWindow.h"
#include "LRS_MainWidget.h"
#include "LRS_Config.h"
#include "LRS_ExpressionParser.h"
//#include "LRS_Debug.h"

#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qevent.h>
#include <qmenu.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

#include <fstream>

namespace LRS {

ScrollWidget::ScrollWidget(
  const std::string& lrsExpr,
  const std::string& configFileName,
  QWidget* parent) : QScrollArea(parent)
{
  Config conf(configFileName);
  
  MainWidget* mainWidget = new MainWidget(
    lrsExpr,
    conf.forKey("fontName", std::string("Lucida Console")),
    conf.forKey("fontSize", 12),
    conf.forKey("minHDistance", 15),
    conf.forKey("minVDistance", 60),
    conf.forKey("borderSize", 10)
    );
  
  QColor bgColor = conf.forKey("bgColor", QColor("#FFFFFF"));
  setPalette(bgColor);

  setAutoFillBackground (true);
  
  setWidget(mainWidget);

  QSize szd = qApp->desktop()->availableGeometry().size();
  int lnw = lineWidth() + 1;
  QSize szw = mainWidget->size() + QSize(lnw * 2, lnw * 2);
  resize(szw.boundedTo(szd));
}

MainWindow::MainWindow(
  const std::string& lrsExpr,
  const std::string& configFileName,
  QWidget* parent) : QMainWindow(parent), _lrsExpr(lrsExpr)
{
  ScrollWidget* w = new ScrollWidget(lrsExpr, configFileName, this);
  setCentralWidget(w);
  
  MainWidget* mw = reinterpret_cast<MainWidget*>(w->widget());
  
  QMenu* menu = menuBar()->addMenu("&File");
  menu->addAction("&Save As...", this, SLOT(slotSave()), Qt::ALT + Qt::Key_S);
  menu->addAction("Save As &XML...", this, SLOT(slotSaveAsXml()), Qt::ALT + Qt::Key_X);
  menu->addAction("&Print", mw, SLOT(print()), Qt::ALT + Qt::Key_P);
  menu->addAction("&Close", this, SLOT(close()), Qt::ALT + Qt::Key_W);
  
  menu = menuBar()->addMenu("&View");
  menu->addAction("Increase &Font Size", mw, SLOT(incFontSize()), Qt::CTRL + Qt::SHIFT + Qt::Key_F);
  menu->addAction("Decrease F&ont Size", mw, SLOT(decFontSize()), Qt::CTRL + Qt::Key_F);
  menu->insertSeparator();
  menu->addAction("Increase &H-Distance", mw, SLOT(incHDistance()), Qt::CTRL + Qt::SHIFT + Qt::Key_H);
  menu->addAction("Decrease H-&Distance", mw, SLOT(decHDistance()), Qt::CTRL + Qt::Key_H);
  menu->insertSeparator();
  menu->addAction("Increase &V-Distance", mw, SLOT(incVDistance()), Qt::CTRL + Qt::SHIFT + Qt::Key_V);
  menu->addAction("Decrease V-D&istance", mw, SLOT(decVDistance()), Qt::CTRL + Qt::Key_V);

  resize(w->size() + QSize(0, menuBar()->height()));
}

static void _SaveString(const std::string& s, const std::string& filename) {
  std::ofstream(filename.c_str()).write(s.c_str(), s.length());
}

void MainWindow::slotSave() {
  QString filename = QFileDialog::getSaveFileName(
    this, "Save As", ".", "LRS files (*.lrs);;All files (*)");
  if (filename.isNull())
    return;
  _SaveString(_lrsExpr, filename.toStdString());
}

void MainWindow::slotSaveAsXml() {
  QString filename = QFileDialog::getSaveFileName(
    this, "Save As XML", ".", "XML files (*.xml);;All files (*)");
  if (filename.isNull())
    return;
  std::string xmlString;
  try {
    LRS::Expression* expr = LRS::ExpressionParser::parse(_lrsExpr);
    xmlString = expr->toXMLString();
    delete expr;
  }
  catch (const LRS::ExpressionParser::SyntaxError& e) {
    QMessageBox::critical(this, "slotSaveAsXml", e.what());
  }
  _SaveString(xmlString, filename.toStdString());
}

}
