/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2006 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @author Martin Lazarov */

#ifndef LRS_TREE_LAYOUT_H
#define LRS_TREE_LAYOUT_H

namespace LRS {
/**
  This class is responsible for positioning the nodes of a given ordered tree.
*/
class TreeLayout {
  int _rootTreePosX, _rootTreePosY, _minNodeHDistance, _minNodeVDistance;
public:
  /** Interface that the nodes handled by the TreeLayout class
      need to implement. */
  struct Node {
    virtual ~Node() {}
    
    /** The number of children of this node */
    virtual unsigned numChildren() = 0;
    /** The node at position p */
    virtual Node* child(unsigned p) = 0;
    
    /** The width of the node */
    virtual int width() = 0;
    /** The height of the node */
    virtual int height() = 0;
    /** The width of the subtree rooted at this node */
    virtual int treeWidth() = 0;
    /** The height of the subtree rooted at this node */
    virtual int treeHeight() = 0;
    /** The x coordinate of the node */
    virtual int x() = 0;
    /** The y coordinate of the node */
    virtual int y() = 0;
    
    /** Sets the position (x,y) of the node */
    virtual void setPosition(int, int) = 0;
    /** Sets the size (width, height) of the subtree rooted at this node */
    virtual void setTreeSize(int, int) = 0;
    
    /** True, iff the node is a leaf */
    bool isTerminal() { return numChildren() == 0; }
  };
  
  /**
    Constructs an instance of TreeLayout.
    @param treePosX the x-position of the tree
    @param treePosY the y-position of the tree
    @param minNodeHDistance the minimum horizontal distance
      between any two nodes
    @param minNodeHDistance the minimum vertical distance
      between any two nodes
  */
  TreeLayout(int treePosX, int treePosY, 
    int minNodeHDistance, int minNodeVDistance);
  
  /** Computes the positions of the nodes of the given tree */
  void layoutNode(Node* node);
  
  /** The minimum horizontal distance used in this TreeLayout. */
  int hDistance() const { return _minNodeHDistance; }
  /** The minimum vertical distance used in this TreeLayout. */
  int vDistance() const { return _minNodeVDistance; }
  
  /** Sets the minimum horizontal distance used in this TreeLayout. */
  void setHDistance(int newValue) { _minNodeHDistance = newValue; }
  /** Sets the minimum vectical distance used in this TreeLayout. */
  void setVDistance(int newValue) { _minNodeVDistance = newValue; }

private:
  void computeTreeSize(Node& node);
  void computeNodePosition(Node& node, int treePosX, int treePosY);
};

}

#endif // !LRS_TREE_LAYOUT_H
