/*  $Id: gdataexchange.cpp,v 1.15 2003/11/22 03:15:22 dm Exp $

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of the GDataExchange class.

    @author Holger Wunsch */


#include <qmessagebox.h>
#include <stdio.h>
#include <qfile.h>
#include <qhostinfo.h>

#include "gdataexchange.h"
#include "tools.h"


/** Creates a new data exchange object. Sets up the server socket.
    @param reader the data reader that handles incoming data. */
GDataExchange::GDataExchange(GAbstractDataReader* reader) {
  inData = NULL;
  outData = NULL;

  dataReader = reader;

  socket = NULL;
}


/** Destroys the data exchange object. */
GDataExchange::~GDataExchange(void) {
  freeResources();
}


void GDataExchange::freeResources() {
  closeServerSocket();
  
  delete inData;
  inData = NULL;
  
  delete outData;
  outData = NULL;
}


/** Finds a free port in the range <code>GDE_PORTRANGE_LO</code> and 
    <code>GDE_PORTRANGE_HI</code> and creates a TCP/IP server socket listening on this
    port. Raises a <code>connectionStateSignal</code> with information
    about the status of the connection, and also prints the status information
    to <code>stdout</code>.
    @return <code>true</code> if the server socket was set up successfully,
      <code>false</code> otherwise. */
bool GDataExchange::setupInetServerSocket(void) {
  port = GDE_PORTRANGE_LO;

  // Scan the port range for a free port
  while (true) {
    if (!serverSocket.listen(QHostAddress::LocalHost, port)) {
      // The port just tried is occupied
      
      if (port == GDE_PORTRANGE_HI) {
        fprintf(stderr, "Opening of the TCP socket connection failed.\n");
        emit connectionStateSignal(GDE_STATE_NO_CONNECTION, NULL, -1);
        return false;
      }
    }
    else
      break;

    port++;
  }

  strcpy(hostname, QHostAddress(QHostAddress::LocalHost).toString().toStdString().c_str());
  
  printf("@|@| %s %d\n", hostname, port);
  fflush(stdout);

  emit connectionStateSignal(GDE_STATE_READY_INET, hostname, port);

  connect(&serverSocket, SIGNAL(newConnection()), SLOT(acceptedSlot()));

  socket = NULL;

  return true;
}


/** Creates a UNIX server socket in the local machine's <code>/tmp</code> directory using
    a unique name. Raises a <code>connectionStateSignal</code> with information
    about the status of the connection, and also prints the status information
    to <code>stdout</code>.
    @return <code>true</code> if the server socket was set up successfully,
      <code>false</code> otherwise. */
bool GDataExchange::setupFileServerSocket(void) {
  return false;
}


/** Closes the server socket. */
void GDataExchange::closeServerSocket(void) {
  // Here it crashes when grale is closed
  // while there's an active connection.
  // The socket is closed when the serversocket
  // is closed anyway, so we should be able to
  // safely skip this part.
  //if (socket)
    //closeSlot(socket);

  if (serverSocket.isListening())
    serverSocket.close();

  if (port == -1)
    QFile::remove(hostname);
  
  emit connectionStateSignal(GDE_STATE_NO_CONNECTION, NULL, -1);
}


/** Called when the server socket accepted a connection.
    @param sock the socket that was created after accepting the connection. */
void GDataExchange::acceptedSlot() {
  QTcpSocket* sock = serverSocket.nextPendingConnection();
  if (socket) {
    qWarning("***GRALE Warning: replacing the existing socket connection");
    socket->close();
  }

  socket  = sock;
  inData  = new GBuffer(GDE_INBOUND_BUFFER_SIZE, GDE_INBOUND_BUFFER_GROW_DELTA);
  outData = new GBuffer(GDE_OUTBOUND_BUFFER_SIZE);

  //socket->enableRead(true);

  QObject::connect(socket, SIGNAL(readyRead()), this, SLOT(readSlot()));
  //QObject::connect(socket, SIGNAL(bytesWritten(qint64)), this, SLOT(bytesWrittenSlot(qint64)));
  QObject::connect(socket, SIGNAL(disconnected()), this, SLOT(disconnectedSlot()));
  
  //QObject::connect(socket, SIGNAL(writeEvent(QTcpSocket*)), this, SLOT(writeSlot(QTcpSocket*)));
  //QObject::connect(socket, SIGNAL(closeEvent(QTcpSocket*)), this, SLOT(closeSlot(QTcpSocket*)));

  emit connectionStateSignal(GDE_STATE_CONNECTED, hostname, port);
}


/** Asynchronously sends out data.
    @param data the data to be sent.
    @param size the size of the data. */
void GDataExchange::asyncSend(unsigned char* data, unsigned int size) {
  if (socket) {
    outData->add(data, size);
  }
}


/** Synchronously sends out data. Does not return before the data has
    been sent. This method performs the same functionality as
    <code>write(...)</code> from the C library.
    @param data the data to be sent
    @param size the size of the data
    @return the number of bytes written, or <code>-1 (0xFFFFFFFF)</code> in case of
    an error (<code>errno</code> can be queried to get additional
    information since this function directly calls <code>write(...)</code>). */
unsigned int GDataExchange::syncSend(unsigned char* data, unsigned int size) {
  if (socket)
    return (unsigned int) socket->write((char*) data, size);
  else
    return (unsigned int) -1;
}


/** Synchronously reads data. If no data is available, this method immediately returns
    (non-blocking I/O).
    @param buffer the buffer the data is to be stored in
    @param size the size of the buffer.
    @return the number of bytes read, or <code>-1 (0xFFFFFFFF)</code> if no data
    is available or in case of an error (<code>errno</code> can be queried to get
    additional information since this function directly calls <code>read(...)</code>). */
unsigned int GDataExchange::syncRead(unsigned char* buffer, unsigned int size) {
  if (socket)
    return (unsigned int) socket->read((char*) buffer, size);
  else
    return (unsigned int) -1;
}


/** Called when data is available for reading. After all available data was read, 
    it is passed on to <code>dataReader->processData</code>.
    @param sock not used. */
void GDataExchange::readSlot() {
  QByteArray buffer = socket->readAll();
  inData->add((unsigned char*) buffer.data(), buffer.length());
  dataReader->processData(inData);
}


/** Called when the socket is ready for writing. Writes all data on <code>outBuffer</code>
    to the socket, then deletes the buffer.
    @param sock not used. */
void GDataExchange::writeSlot(QTcpSocket* sock) {
  socket->write((const char*) outData->getPtr(), outData->getSize());

  delete outData;
  outData = new GBuffer(GDE_OUTBOUND_BUFFER_SIZE);
}

void GDataExchange::disconnectedSlot() {
  socket = NULL;

  delete inData;
  inData = NULL;

  delete outData;
  outData = NULL;
  
  emit connectionStateSignal(GDE_STATE_READY_INET, hostname, port);
}

