/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Implementation of class <code>GDataHolder</code>.

    @author Holger Wunsch */


#include <qmessagebox.h>
//Added by qt3to4:
#include <Q3PtrList>

#include "gdataholder.h"
#include "tools.h"
#include "global.h"


/** Creates a new data holder that contains a feature structure.
    @param data the feature structure data package. */
GDataHolder::GDataHolder(GStructureData* data) {
  this->data = data;
  datatype = DT_FEATURE_STRUCTURE;

  viewers = new Q3PtrList<GDataViewer>;
}


/** Creates a new data holder that contains a type hierarchy.
    @param data the type hierarchy data package. */
GDataHolder::GDataHolder(GTypeHierarchyData* data) {
  this->data = data;
  datatype = DT_TYPE_HIERARCHY;

  viewers = new Q3PtrList<GDataViewer>;
}


GDataHolder::~GDataHolder(void) {
  closeAllViewers();
  delete viewers;
  viewers = NULL;

  switch (datatype) {
  case DT_FEATURE_STRUCTURE:
    delete (GStructureData*) data;
    break;
  case DT_TYPE_HIERARCHY:
    delete (GTypeHierarchyData*) data;
    break;
  }
}


/** Returns the type of the data package contained in this holder.
    @return the type of the data package. */
int GDataHolder::type(void) {
  return datatype;
}


/** Returns the name of the data package. */
const QString GDataHolder::getName(void) {
  QString name;

  switch (datatype) {
  case DT_FEATURE_STRUCTURE:
    name = featureStructure()->getName();
    break;
  case DT_TYPE_HIERARCHY:
    name = typeHierarchy()->getName();
    break;
  }

  return name;
}
    
    
/** Returns a feature structure data package.
    @return a pointer to the feature structure data package, or <code>NULL</code>
    if the data package contained in this holder is not a feature structure. */
GStructureData* GDataHolder::featureStructure(void) {
  if (datatype != DT_FEATURE_STRUCTURE)
    return NULL;

  return (GStructureData*) data;
}


/** Returns a type hierarchy data package.
    @return a pointer to the type hierarchy data package, or <code>NULL</code>
    if the data package contained in this holder is not a type hierarchy. */
GTypeHierarchyData* GDataHolder::typeHierarchy(void) {
  if (datatype != DT_TYPE_HIERARCHY)
    return NULL;

  return (GTypeHierarchyData*) data;
}
  

/** Opens a viewer for the data package
    @param parent the parent widget.
    @param showFlags data package dependent information about how the data is shown.

    For feature structures, this can be a <code>DRAW_xxxx</code> value. */
GDataViewer* GDataHolder::openViewer(QWidget* parent, int showFlags) {
  GDataViewer* viewer = 0;

  switch (datatype) {
  case DT_FEATURE_STRUCTURE:
    viewer = openFeatureStructureViewer(parent, showFlags);
    break;
  case DT_TYPE_HIERARCHY:
    viewer = openTypeHierarchyViewer(parent, showFlags);
    break;
  }

  viewer->show();

  viewers->append(viewer);

  return viewer;
}


/** Opens a viewer for feature structures.
    @param parent the parent widget.
    @param showFlags if <code>DRAW_AUTO</code> is passed in this argument, a tree will be
    displayed initially if tree data is available, otherwise a feature structure is displayed.
    <code>DRAW_STRUC</code> displays a feature structure, <code>DRAW_TREE</code> displays
    a tree.
    @return the new viewer. */
ViewCanvas* GDataHolder::openFeatureStructureViewer(QWidget* parent, int showFlags) {
  GStructureData* fs = featureStructure();

  if (showFlags == DRAW_AUTO) {
    if (fs->tree != NULL)
      showFlags = DRAW_TREE;
    else
      showFlags = DRAW_STRUC;
  }

  ViewCanvas* viewer = new ViewCanvas(featureStructure(), showFlags, globalOptions, this, parent);

  viewer->show();
  viewer->redraw();

  return viewer;
}


/** Opens a viewer for type hierarchies.
    @param parent the parent widget.
    @param showFlags currently not used for type hierarchies.
    @return the new viewer. */
GSVGWindow* GDataHolder::openTypeHierarchyViewer(QWidget* parent, int showFlags) {
  GSVGWindow* viewer = new GSVGWindow(typeHierarchy()->getFilename(), this, parent);

  return viewer;
}


/** Closes a viewer.
    @param index the index of the viewer. */
void GDataHolder::closeViewer(int index) {
  viewers->at(index)->close(true);
}


/** Closes all viewers. */
void GDataHolder::closeAllViewers(void) {
  Q3PtrList<GDataViewer> viewersCopy(*viewers);

  for (int i = viewersCopy.count() - 1; i >= 0; i--)
    viewersCopy.at(i)->close(true);
}


/** Returns the number of open viewers for this data package.
    @return the number of open viewers. */
int GDataHolder::numViewers(void) {
  if (viewers)
    return viewers->count();
  else
    return 0;
}


/** Checks if a given viewer is belongs to this data holder.
    @param viewer the viewer to be checked
    @return <code>true</code> if the viewer belongs to this data holder,
    <code>false</code> otherwise. */
bool GDataHolder::hasViewer(GDataViewer* viewer) {
  bool contains = false;
  GDataViewer* other = viewers->first();

  while (other != NULL) {
    if (other == viewer) {
      contains = true;
      break;
    }
    
    other = viewers->next();
  }

  return contains;
}
      

/** Saves the data package. This method supports saving of multiple
    data packages in one shot. 

    This method displays a "Save As" dialog if the <code>basename</code>
    argument is <code>NULL</code>. In this case, only the data package
    contained in this holder will be saved. The dialog will display the
    right choice of file extensions according to the data type of the
    data package to be saved.

    When the <code>basename</code> is non-null, this method will name
    the file by appending <code>number</code> to <code>basename</code>,
    then adding the default extension for the data package.
    @param parent the parent widget.
    @param basename the basic filename when used while saving multiple
      data packages, or <code>NULL</code>, if only this data package is
      to be saved.
    @param number the number of this data package. The value of
      <code>number</code> is ignored if <code>basename</code> is <code>NULL</code>. */
void GDataHolder::save(QWidget* parent, const QString& basename, int number) {
  switch (datatype) {
  case DT_FEATURE_STRUCTURE:
    saveFeatureStructure(parent, basename, number);
    break;
  case DT_TYPE_HIERARCHY:
    saveTypeHierarchy(parent, basename, number);
    break;
  }
}


/** Saves a feature structure. See also <code>save</code>.
    @param parent the parent widget.
    @param basename the basic filename.
    @param number the number of this data package. */
void GDataHolder::saveFeatureStructure(QWidget* parent, const QString& basename, int number) {
  QString filename;

  if (basename.isNull()) {
    filename = getSaveFilename(parent, "Save Data", true, "GRALE data files (*.GRALE);;All files (*)");
    
    if (filename.isNull())
      return;
  }
  else
    filename.sprintf("%s%d.GRALE", (const char*) basename.local8Bit(), number);

  FILE* file = fopen((const char*) filename.local8Bit(), "w");
  
  if (file) {
    parent->setCursor(Qt::waitCursor);
    
    featureStructure()->write(file);
    fprintf(file, "\n");
    fclose(file);
    
    parent->setCursor(Qt::arrowCursor);
  }
  else {
    char buf[500];
    sprintf(buf, "Could not open file %s", (const char*) filename.local8Bit());
    QMessageBox::critical(parent, "Save Data", buf);
  }
}


/** Saves a type hierarchy. See also <code>save</code>.
    @param parent the parent widget.
    @param basename the basic filename.
    @param number the number of this data package. */
void GDataHolder::saveTypeHierarchy(QWidget* parent, const QString& basename, int number) {
  QString filename;

  if (basename.isNull()) {
    filename = getSaveFilename(parent, "Save Data", true, "GRALE data files (*.GRALE);;All files (*)");
    
    if (filename.isNull())
      return;
  }
  else
    filename.sprintf("%s%d.GRALE", (const char*) basename.local8Bit(), number);

  FILE* file = fopen((const char*) filename.local8Bit(), "w");
  
  if (file) {
    parent->setCursor(Qt::waitCursor);
    
    typeHierarchy()->write(file);
    fprintf(file, "\n");
    fclose(file);
    
    parent->setCursor(Qt::arrowCursor);
  }
  else {
    char buf[500];
    sprintf(buf, "Could not open file %s", (const char*) filename.local8Bit());
    QMessageBox::critical(parent, "Save Data", buf);
  }
}


/** Removes a viewer that was just closed from the list of currently open viewers.
    @param viewer a pointer to the viewer to be removed. */
void GDataHolder::viewerClosed(GDataViewer* viewer) {
  for (unsigned int i = 0; i < viewers->count(); i++)
    if (viewers->at(i) == viewer) {
      viewers->remove(i);
      break;
    }

  mainw->viewerClosed(viewer);
}
