/*  $Id$

    This file is part of the

    GRALE: a grammar visualization tool

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2005 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @file

   Declarations for the <code>GDataManager</code> class.

   @author Holger Wunsch */


#ifndef __GDATAMANAGER_H
#define __GDATAMANAGER_H


#include <q3ptrlist.h>
#include <q3valuevector.h>

#include "gdataholder.h"
#include "gabstractdatareader.h"


/** This class implements a general data manager. GRALE can handle several different
    kinds of data, among them feature structure data, SVG graphics containing type hierarchies,
    etc. This data manager encapsulates the supported operations on the data and provides
    interfaces to UI features. 

    @author Holger Wunsch */
class GDataManager: public QObject, public GAbstractDataReader {
  Q_OBJECT;


  /** A list of data packages currently available. */
  Q3PtrList<GDataHolder>* data;


  /** The parent widget. See also <code>GDataManager()</code>. */
  QWidget* parent;


  /** Reads a data package and creates a new document from it.
      Depending on the user setting, a new viewer is opened.
      @param line the line containing the data package. */
  void parseData(char* line);


 public:
  /** Creates a new empty data manager.
      @param parent pointer to a widget which will be the parent widget of dialog boxes
      displayed by the data manager, as well as the receiver of update messages. */
  GDataManager(QWidget* parent);


  /** Destroys all data packages managed by this data manager and this data
      manager itself. */
  ~GDataManager(void);


  /** Inserts a new data package in the data manager.
      @param newData the new data package. */
  void addData(GDataHolder* newData);


  /** Returns the data package indexed by <code>index</code>.
      @param index the index. */
  GDataHolder* getData(int index);


  /** Deletes a data package.
      @param index the index of the data package to be deleted.
      @param notify if this argument is <code>true</code>, a <code>dataModifiedSignal()</code>
      will be emitted. If <code>false</code>, no signal will be emitted. */
  void deleteData(int index, bool notify);


  /** Deletes the data packages index by <code>indices</code>.
      @param indices the indices of the data packages to be deleted. */
  void deleteData(Q3ValueVector<int>& indices);


  /** Deletes all data packages. */
  void deleteAllData(void);


  /** Returns the number of available data packages. */
  int numData(void);

  
  /** Displays a file dialog which allows the user to pick a file; the data
      contained in that file is loaded afterwards.
      @param parent the parent widget. */
  void loadData(QWidget* parent);


  /** Loads a GRALE data file and creates a new view window, depending on the user-settings.
      @param fileName the name of the data file. */
  void loadDataFile(const QString& fileName);


  /** Saves a data package to a file. 
      @param index the index of the data package to be saved. */
  void saveData(int index);


  /** Saves multiple data packages at once.
      @param indices a vector of the indices of the data packages to be saved. */
  void saveData(Q3ValueVector<int>& indices);


  /** This method is called by an instance of <code>GDataExchange</code> when
      new data has become available.
      
      It is not guaranteed that <code>buffer</code> remains valid after a call
      to this method. */
  virtual void processData(GBuffer* buffer);


  /** Returns the total number of open viewers (that is, all open viewers for
      all available data packages.
      @return the number of open viewers. */
  int numViewers(void);


  /** Closes all open viewers. */
  void closeAllViewers(void);


 signals:
  /** Raised when a data package was added or removed. */
  void dataModifiedSignal(void);


  /** Raised when a viewer is opened or closed. */
  void viewerSignal(void);


 public slots:
   /** Deletes this instance. This should be connected to <code>KApplication::shutDown()</code>,
       such that viewer windows can be properly deleted. Viewer windows, which are opened by
       instances of <code>GDataHolder</code>, are created as children of the application's main window, and
       deleted when the application shuts down. However, a data holder will also delete its
       dependent viewer windows when it is deleted itself. This would cause a double deletion of
       the viewer windows if the data holder instances are deleted after the main window is
       deleted. Since viewer windows will automatically deregister themselves from their
       parent window when deleted prior to the main window, all data holders and dependent
       viewers <b>must</b> be deleted first. By connecting this slot to 
       <code>KApplication::shutdown()</code>, this requirement can be fulfilled. */
   void deleteSlot(void);
};


#endif
