% $Id: unfill.pl,v 1.10 2003/04/22 17:00:50 dm Exp $
%
% This file defines the predicate get_unfill/3, which returns
% an assoc list with the ref variables of each node which are
% neither informative themselves, nor any of its substructures. 
%
% A local node is classified as informative if
% a) its type value is more specific than the appropriate value, or
% b) the node is structure shared
%
% Author: Detmar Meurers
% 
% ------------------------------------------------------

:- use_module(library(assoc),[assoc_to_list/2,put_assoc/4,empty_assoc/1,
                              get_assoc/3]).

% ----------------------------------------------------
% switches:

:- dynamic unfill_flag/1.

unfill_on :-
   asserta(unfill_flag(true)).

unfill_off :-
   asserta(unfill_flag(fail)).

unfill_restore :-    % restore to previous value
   retract(unfill_flag(_)).

unfill_flag :-	     % 0-arity pred for asking the value of this flag
   unfill_flag(X), !,
   call(X).

:- unfill_on.

% ----------------------------------------------------
% top-level predicate get_unfill/3:

get_unfill(FS,Tags,Unf0,Unf) :-
   unfill_flag, !,
   empty_assoc(Empty),
   get_unfill_act(FS,bot,Tags,Empty,_Vis,Unf0,Unf,_,_).
get_unfill(_,_,Unf,Unf).

get_unfill_act(FS,AppropT,Tags,Vis0,Vis,Unf0,Unf,Inf0,Inf) :-
   % deref(FS,Ref,SVs),
   (   get_assoc(FS,Vis0,_)	   % Seen this node before?
   ->  Vis=Vis0,Unf=Unf0	   % Yes: nothing to do
   ;   put_assoc(FS,Vis0,_,Vis1), % No: store we've now seen it and
       get_type(FS,Type),
       approps(Type,FRs,_),
       build_keyed_feats(FRs,FS,KeyedFeats),
       get_unfill_local(Type,AppropT,FS,Tags,Inf0,Inf1),
       get_unfill_feats(KeyedFeats,Tags,Vis1,Vis,Unf0,Unf1,Inf1,Inf),
       mark_unfill(FS,Inf0,Inf,Unf1,Unf)
   ).

% mark_unfill/5: if neither the node itself nor any of its 
% substructures was informative, mark this node as to be unfilled
mark_unfill(FS,Inf0,Inf,Unf0,Unf) :-
   (   Inf0==Inf
   ->  put_assoc(FS,Unf0,_,Unf)
   ;   Unf=Unf0
   ).

get_unfill_feats([],_Tags,Vis,Vis,Unf,Unf,Inf,Inf).
get_unfill_feats([fval(_F,V,AppropT)|Vs],Tags,Vis0,Vis,Unf0,Unf,Inf0,Inf) :-
   get_unfill_act(V,AppropT,Tags,Vis0,Vis1,Unf0,Unf1,Inf0,Inf1),
   get_unfill_feats(Vs,Tags,Vis1,Vis,Unf1,Unf,Inf1,Inf).

get_unfill_local(Type,AppropT,FS,Tags,Inf0,Inf) :-
   (   locally_uninformative(Type,AppropT,FS,Tags)
   ->  Inf=Inf0
   ;   Inf=[true|Inf0]
   ).

% A node is taken to be locally uninformative if:
%
% a) An a_ atom is the appropriate value and only a variable
%    is provided as argument of the a_ atom
locally_uninformative((a_ Arg),(a_ _),_,_) :-
   !,
   var(Arg).

% a-1/2) ALE says it's uninformative (GBP)
locally_uninformative(0,_,_,_) :-
  !.

% b) The type is identical to the appropriate value, 
%    and the node is not structure shared.
locally_uninformative(Type,Type,FS,Tags) :-
   \+ get_assoc(FS,Tags,_).


%%           -*-Mode: prolog;-*-

